@@ -61,6 +61,10 @@ else:
 
 _HEADER_EXTRA_CREDENTIAL_KEY_REGEX = re.compile(r'^\S[^\s=]*$')
 
+INF = float("inf")
+NEGATIVE_INF = float("-inf")
+NAN = float("nan")
+
 
 class ClientSession(object):
     def __init__(
@@ -73,6 +77,7 @@ class ClientSession(object):
         headers=None,
         transaction_id=None,
         extra_credential=None,
+        client_tags=None
     ):
         self.catalog = catalog
         self.schema = schema
@@ -84,6 +89,7 @@ class ClientSession(object):
         self._headers = headers or {}
         self.transaction_id = transaction_id
         self.extra_credential = extra_credential
+        self.client_tags = client_tags
 
     @property
     def properties(self):
@@ -107,12 +113,13 @@ def get_session_property_values(headers, header):
 
 
 class TrinoStatus(object):
-    def __init__(self, id, stats, warnings, info_uri, next_uri, rows, columns=None):
+    def __init__(self, id, stats, warnings, info_uri, next_uri, update_type, rows, columns=None):
         self.id = id
         self.stats = stats
         self.warnings = warnings
         self.info_uri = info_uri
         self.next_uri = next_uri
+        self.update_type = update_type
         self.rows = rows
         self.columns = columns
 
@@ -221,7 +228,8 @@ class TrinoRequest(object):
         max_attempts: int = MAX_ATTEMPTS,
         request_timeout: Union[float, Tuple[float, float]] = constants.DEFAULT_REQUEST_TIMEOUT,
         handle_retry=exceptions.RetryWithExponentialBackoff(),
-        verify: bool = True
+        verify: bool = True,
+        client_tags: Optional[List[str]] = None
     ) -> None:
         self._client_session = ClientSession(
             catalog,
@@ -232,6 +240,7 @@ class TrinoRequest(object):
             http_headers,
             transaction_id,
             extra_credential,
+            client_tags
         )
 
         self._host = host
@@ -281,6 +290,8 @@ class TrinoRequest(object):
         headers[constants.HEADER_SCHEMA] = self._client_session.schema
         headers[constants.HEADER_SOURCE] = self._client_session.source
         headers[constants.HEADER_USER] = self._client_session.user
+        if self._client_session.client_tags is not None and len(self._client_session.client_tags) > 0:
+            headers[constants.HEADER_CLIENT_TAGS] = ",".join(self._client_session.client_tags)
 
         headers[constants.HEADER_SESSION] = ",".join(
             # ``name`` must not contain ``=``
@@ -448,6 +459,7 @@ class TrinoRequest(object):
             warnings=response.get("warnings", []),
             info_uri=response["infoUri"],
             next_uri=self._next_uri,
+            update_type=response.get("updateType"),
             rows=response.get("data", []),
             columns=response.get("columns"),
         )
@@ -522,8 +534,31 @@ class TrinoResult(object):
                     "typeSignature": data_type["typeSignature"]["arguments"][0]["value"]
                 }
                 return [cls._map_to_python_type((array_item, raw_type)) for array_item in value]
+            if isinstance(value, dict):
+                if raw_type == "map":
+                    raw_key_type = {
+                        "typeSignature": data_type["typeSignature"]["arguments"][0]["value"]
+                    }
+                    raw_value_type = {
+                        "typeSignature": data_type["typeSignature"]["arguments"][1]["value"]
+                    }
+                    return {
+                        cls._map_to_python_type((key, raw_key_type)):
+                            cls._map_to_python_type((value[key], raw_value_type))
+                        for key in value
+                    }
+                # TODO: support row type
+                return value
             elif "decimal" in raw_type:
                 return Decimal(value)
+            elif raw_type == "double":
+                if value == 'Infinity':
+                    return INF
+                elif value == '-Infinity':
+                    return NEGATIVE_INF
+                elif value == 'NaN':
+                    return NAN
+                return value
             elif raw_type == "date":
                 return datetime.strptime(value, "%Y-%m-%d").date()
             elif raw_type == "timestamp with time zone":
@@ -572,6 +607,7 @@ class TrinoQuery(object):
         self._finished = False
         self._cancelled = False
         self._request = request
+        self._update_type = None
         self._sql = sql
         self._result = TrinoResult(self, experimental_python_types=experimental_python_types)
         self._response_headers = None
@@ -590,6 +626,10 @@ class TrinoQuery(object):
     def stats(self):
         return self._stats
 
+    @property
+    def update_type(self):
+        return self._update_type
+
     @property
     def warnings(self):
         return self._warnings
@@ -627,6 +667,7 @@ class TrinoQuery(object):
 
     def _update_state(self, status):
         self._stats.update(status.stats)
+        self._update_type = status.update_type
         if status.columns:
             self._columns = status.columns
 
